/*
 * External interface to the bcm_lzma subsystem
 *
 * This is derived from the 2.6.8.1 kernel patch available at
 * http://www.efixo.com/neufbox4/freesoftware/archives/kernel.patch,
 * Which doesn't bear any copyright notice.
 *
 * Changes to this file are 
 * Copyright (c) 2008 Simon Vallet <svallet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * 7zlzma.c
 */
#include <7zapi.h>

#include "LZMADecoder.h"

/* Those are referenced in IInOutStreams.h */
ISequentialInStream  in_stream;
ISequentialOutStream out_stream;

/**
 * @return the size of the decompressed stream if decompression was successful
 *         a negative error code otherwise
 */
ssize_t decompress_lzma_7zapi (unsigned char* in_data, unsigned int in_size, 
			       unsigned char* out_data, unsigned int out_size)
{

  LzmaDecoder cc; int RC;
  UINT64 in_size_l  = in_size;
  UINT64 out_size_l = out_size;
  
  InStreamInit(in_data, in_size);
  OutStreamInit((char *)out_data, out_size);
  
  LzmaDecoderConstructor(&cc);

  if ((RC = LzmaDecoderReadCoderProperties(&cc)) != S_OK) {
    return RC;
  }
  
  if (LzmaDecoderCode(&cc, &in_size_l, &out_size_l) != S_OK) {
    return -2;
  }
  
  if (out_stream.size != out_size) {
    return -3;
  }
  
  if (out_stream.overflow) {
    return -4;
  }
  
  return out_stream.total;
}



