/*
 * User-space lzma_inflate implementation
 *
 * This is derived from the 2.6.8.1 kernel patch available at
 * http://www.efixo.com/neufbox4/freesoftware/archives/kernel.patch,
 * Which doesn't bear any copyright notice.
 *
 * Changes to this file are 
 * Copyright (c) 2008 Simon Vallet <svallet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * AriBitCoder.h
 */
#ifndef __COMPRESSION_BITCODER_H
#define __COMPRESSION_BITCODER_H

#include "RangeCoder.h"

#define kNumBitModelTotalBits  11
#define kBitModelTotal (1 << kNumBitModelTotalBits)

#define kNumMoveReducingBits 2


typedef UINT32 CBitDecoder;

INLINE void BitDecoderInit(CBitDecoder *bitDecoder)
{ 
  *bitDecoder = kBitModelTotal / 2; 
}

INLINE UINT32 BitDecode(ISequentialInStream *in_stream, CBitDecoder *bitDecoder, CRangeDecoder *aRangeDecoder)
{
 
  UINT32 aNewBound = (aRangeDecoder->m_Range >> kNumBitModelTotalBits) * (*bitDecoder);
  if (aRangeDecoder->m_Code < aNewBound) {
    aRangeDecoder->m_Range = aNewBound;
    *bitDecoder += (kBitModelTotal - *bitDecoder) >> kNumMoveBits;
    
    if (aRangeDecoder->m_Range < kTopValue) {
      aRangeDecoder->m_Code = (aRangeDecoder->m_Code << 8) | InStreamReadByte(in_stream);
      aRangeDecoder->m_Range <<= 8;
    }
    
    return 0;
  
  } else {

    aRangeDecoder->m_Range -= aNewBound;
    aRangeDecoder->m_Code -= aNewBound;
    *bitDecoder -= (*bitDecoder) >> kNumMoveBits;
    
    if (aRangeDecoder->m_Range < kTopValue) {
      aRangeDecoder->m_Code = (aRangeDecoder->m_Code << 8) | InStreamReadByte(in_stream);
      aRangeDecoder->m_Range <<= 8;
    }

    return 1;
  }
}

#endif


