/*
 * User-space lzma_inflate implementation
 *
 * This is derived from the 2.6.8.1 kernel patch available at
 * http://www.efixo.com/neufbox4/freesoftware/archives/kernel.patch,
 * Which doesn't bear any copyright notice.
 *
 * Changes to this file are 
 * Copyright (c) 2008 Simon Vallet <svallet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * LZMADecoder.h
 */
#ifndef __LZARITHMETIC_DECODER_H
#define __LZARITHMETIC_DECODER_H

#include "WindowOut.h"
#include "LZMA.h"
#include "LenCoder.h"
#include "LiteralCoder.h"


typedef struct LzmaDecoder
{
  CRangeDecoder m_RangeDecoder;

  CBitDecoder m_MainChoiceDecoders[kNumStates][kNumPosStatesMax];
  CBitDecoder m_MatchChoiceDecoders[kNumStates];
  CBitDecoder m_MatchRepChoiceDecoders[kNumStates];
  CBitDecoder m_MatchRep1ChoiceDecoders[kNumStates];
  CBitDecoder m_MatchRep2ChoiceDecoders[kNumStates];
  CBitDecoder m_MatchRepShortChoiceDecoders[kNumStates][kNumPosStatesMax];

  CBitTreeDecoder               m_PosSlotDecoder[kNumLenToPosStates];

  CReverseBitTreeDecoder2       m_PosDecoders[kNumPosModels];
  CReverseBitTreeDecoder        m_PosAlignDecoder;
  
  LenDecoder m_LenDecoder;
  LenDecoder m_RepMatchLenDecoder;

  LitDecoder m_LiteralDecoder;

  UINT32 m_DictionarySize;
  
  UINT32 m_PosStateMask;
} LzmaDecoder;

HRESULT LzmaDecoderCreate (LzmaDecoder *lzmaDecoder);
HRESULT LzmaDecoderInit (LzmaDecoder *lzmaDecoder);
HRESULT LzmaDecoderCodeReal (LzmaDecoder *lzmaDecoder, UINT64 *anInSize, UINT64 *anOutSize);

void LzmaDecoderConstructor (LzmaDecoder *lzmaDecoder);

HRESULT LzmaDecoderCode (LzmaDecoder *lzmaDecoder, UINT64 *anInSize, UINT64 *anOutSize);
HRESULT LzmaDecoderReadCoderProperties (LzmaDecoder *lzmaDecoder);
HRESULT LzmaDecoderSetDictionarySize (LzmaDecoder *lzmaDecoder, UINT32 aDictionarySize);
HRESULT LzmaDecoderSetLiteralProperties (LzmaDecoder *lzmaDecoder, UINT32 aLiteralPosStateBits, UINT32 aLiteralContextBits);
HRESULT LzmaDecoderSetPosBitsProperties (LzmaDecoder *lzmaDecoder, UINT32 aNumPosStateBits);

#endif
