/*
 * User-space lzma_inflate implementation
 *
 * This is derived from the 2.6.8.1 kernel patch available at
 * http://www.efixo.com/neufbox4/freesoftware/archives/kernel.patch,
 * Which doesn't bear any copyright notice.
 *
 * Changes to this file are 
 * Copyright (c) 2008 Simon Vallet <svallet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * LenCoder.h
 */
#ifndef __LENCODER_H
#define __LENCODER_H

#include "BitTreeCoder.h"

#define kNumPosStatesBitsMax 4
#define kNumPosStatesMax 16


#define kNumPosStatesBitsEncodingMax 4
#define kNumPosStatesEncodingMax 16

#define kNumLenBits 3
#define kNumLowSymbols (1 << kNumLenBits)

#define kNumMidBits 3
#define kNumMidSymbols (1 << kNumMidBits)

#define kNumHighBits 8

#define kNumSymbolsTotal (kNumLowSymbols + kNumMidSymbols + (1 << kNumHighBits))

typedef struct LenDecoder
{
  CBitDecoder       m_Choice;
  CBitDecoder       m_Choice2;
  CBitTreeDecoder   m_LowCoder[kNumPosStatesMax];
  CBitTreeDecoder   m_MidCoder[kNumPosStatesMax];
  CBitTreeDecoder   m_HighCoder; 
  UINT32            m_NumPosStates;
} LenDecoder;

INLINE void LenDecoderCreate(LenDecoder *lenCoder, UINT32 aNumPosStates)
{ 
  lenCoder->m_NumPosStates = aNumPosStates; 
}

INLINE void LenDecoderInit(LenDecoder *lenCoder)
{
  UINT32 aPosState;
  BitDecoderInit(&lenCoder->m_Choice);
  for (aPosState = 0; aPosState < lenCoder->m_NumPosStates; aPosState++) {
    BitTreeDecoderInit(&lenCoder->m_LowCoder[aPosState],kNumLenBits);
    BitTreeDecoderInit(&lenCoder->m_MidCoder[aPosState],kNumMidBits);
  }
  BitTreeDecoderInit(&lenCoder->m_HighCoder,kNumHighBits);
  BitDecoderInit(&lenCoder->m_Choice2);
}

INLINE UINT32 LenDecode(ISequentialInStream *in_stream, LenDecoder *lenCoder, CRangeDecoder *aRangeDecoder, UINT32 aPosState)
{

  if(BitDecode(in_stream, &lenCoder->m_Choice, aRangeDecoder) == 0) {
    return BitTreeDecode(in_stream, &lenCoder->m_LowCoder[aPosState],aRangeDecoder);
  } else {

      UINT32 aSymbol = kNumLowSymbols;

      if(BitDecode(in_stream, &lenCoder->m_Choice2, aRangeDecoder) == 0) {
        aSymbol += BitTreeDecode(in_stream, &lenCoder->m_MidCoder[aPosState],aRangeDecoder);
      } else {
        aSymbol += kNumMidSymbols;
        aSymbol += BitTreeDecode(in_stream, &lenCoder->m_HighCoder,aRangeDecoder);
      }

      return aSymbol;
  }
}

#endif
