/*
 * User-space lzma_inflate implementation
 *
 * This is derived from the 2.6.8.1 kernel patch available at
 * http://www.efixo.com/neufbox4/freesoftware/archives/kernel.patch,
 * Which doesn't bear any copyright notice.
 *
 * Changes to this file are 
 * Copyright (c) 2008 Simon Vallet <svallet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * RangeCoder.h
 */
#ifndef __COMPRESSION_RANGECODER_H
#define __COMPRESSION_RANGECODER_H

#include "IInOutStreams.h"

#define kNumTopBits 24
#define kTopValue (1 << kNumTopBits)

typedef struct CRangeDecoder
{
  UINT32  m_Range;
  UINT32  m_Code;
} CRangeDecoder;


INLINE void RangeDecoderInit(ISequentialInStream *in_stream, CRangeDecoder *rangeDecoder)
{
  int i;
  rangeDecoder->m_Code = 0;
  rangeDecoder->m_Range = (UINT32)(-1);
  for(i = 0; i < 5; i++) {
      rangeDecoder->m_Code = (rangeDecoder->m_Code << 8) | InStreamReadByte(in_stream);
  }
}

INLINE UINT32 RangeDecodeDirectBits(ISequentialInStream *in_stream, CRangeDecoder *rangeDecoder, UINT32 aNumTotalBits)
{
  UINT32 aRange = rangeDecoder->m_Range;
  UINT32 aCode = rangeDecoder->m_Code;        
  UINT32 aResult = 0;
  UINT32 i;
 
  for (i = aNumTotalBits; i > 0; i--) {
    UINT32 t;
    aRange >>= 1;
    t = (aCode - aRange) >> 31;
    aCode -= aRange & (t - 1);
    aResult = (aResult << 1) | (1 - t);

    if (aRange < kTopValue) {
      aCode = (aCode << 8) | InStreamReadByte(in_stream);
      aRange <<= 8; 
    }
  }
   
  rangeDecoder->m_Range = aRange;
  rangeDecoder->m_Code = aCode;
  return aResult;
}

#endif
