/*
 * Utility functions the image tools.
 *
 * Copyright (c) Simon Vallet 2008 <svallet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * bcm_img.h
 */

/* OpenWrt constants */
#include "bcmTag.h"


/* The in-memory representation of the Netgear header */
struct ng_bcm_hdr {
  unsigned int version;                    /* tagVersion */
  unsigned char vendor_str_1[SIG_LEN];     /* signiture_1*/
  unsigned char vendor_str_2[SIG_LEN_2];   /* signiture_2*/
  unsigned char chip_id[CHIP_ID_LEN];      /* chipId */
  unsigned char board_id[BOARD_ID_LEN];    /* boardId */
  unsigned char reserved[RESERVED_LEN];    /* reserved */
  unsigned char is_big_endian;             /* 1 is BE, 0 is LE */
  unsigned char is_dual_img;               /* 1 is dual, 0 is not */
  unsigned char is_inactive;               /* 1 is inactive, 0 is not */
  off_t cfe_addr;                          /* cfeAddress */
  off_t rootfs_addr;                       /* rootfsAddress */
  off_t kernel_addr;                       /* kernelAddress */
  size_t img_sz;                           /* totalImageLen */
  size_t cfe_sz;                           /* cfeLen */
  size_t rootfs_sz;                        /* rootfsLen */
  size_t kernel_sz;                        /* kernelLen */
  u_int32_t hdr_chksum;                    /* tagValidationToken */
  u_int32_t img_chksum;                    /* imageValidationToken */
  u_int32_t rk_chksum;                     /* checksum of (rootfs||kernel) */
};

/* The in-memory representation of the kernel header */
struct ng_kernel_hdr {
  u_int32_t load_base;
  u_int32_t entry_point;
  u_int32_t compressed_sz;
};

/* File names */
#define _HDR_FILE "header.bin"
#define _BOOTLDR_FILE "bootldr.bin"
#define _ROOTFS_FILE "rootfs.bin"

/* Default offsets */
#define _DFLT_BOOTLDR_OFF 0xbfc00000
#define _DFLT_ROOTFS_OFF  0xbfc10100
/* No default for kernel */

#define _DFLT_BOARD_ID "RTA1320"  /* Default board id */
#define _DFLT_CHIP_ID  "6338"     /* Default chip id */

/* Offset of the unknown byte in the reserved field */
#define _RSRVD_UB_OFFSET  50
/* Offset of the (rootfs||kernel) CRC in the reserved field */
#define _RSRVD_RK_CRC_OFFSET 54 

/* File names for kernel image manipulation */
#define _DFLT_RK_FILE "vmlinux.bin"
#define _DFLT_EK_FILE "vmlinux"
#define _DFLT_ZK_FILE "vmlinux.lz"
#define _DFLT_KIMG_FILE "vmlinux.img"

/* Length of the kernel image header */
#define _KIMG_TAG_LEN 12

/* lzma_compress args */
#define _KIMG_LZMA_ALG 2
#define _KIMG_LZMA_DICTSZ (1 << 22)
#define _KIMG_LZMA_FASTBYTES 128

/* Kernel header params */
#define _KIMG_DFLT_LOAD_ADDR 0x80010000U
#define _KIMG_DFLT_LOAD_ADDR_SWAP 0x00000180U
