/*
 * Unsquash a squashfs filesystem.  This is a highly compressed read only filesystem.
 *
 * Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007
 *  Phillip Lougher <phillip@lougher.demon.co.uk>
 *
 * This file has been modified to support LZMA decompression as 
 * implemented in Broadcom chipsets -- most of the code is shamelessly 
 * borrowed from nb4-unsquash.c available at
 * svn://svn.gna.org/svn/openbox4/trunk/tools/nb4-unsquash openbox4/trunk/tools/nb4-unsquash
 *
 * Changes to this file are
 * Copyright (c) Simon Vallet <svallet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * lzma_unsquash.h
 */
#ifndef _LZMA_UNSQUASH_H
#define _LZMA_UNSQUASH_H

#include <regex.h>

#include <squashfs_fs.h>
#include <read_fs.h>
#include <global.h>
#include <7zapi.h>

#ifdef __GNUC__
  #include <endian.h>
#else
  #define __BYTE_ORDER BYTE_ORDER
  #define __BIG_ENDIAN BIG_ENDIAN
  #define __LITTLE_ENDIAN LITTLE_ENDIAN
#endif

#ifdef __CYGWIN__
  #define FNM_EXTMATCH (1 << 5)
  #define SYSTEM_UID 18
#else
  #define SYSTEM_UID 0
#endif

#ifdef SQUASHFS_TRACE
  #define TRACE(s, args...)		do { printf("unsquashfs: "s, ## args); } while(0)
#else
  #define TRACE(s, args...)
#endif

#define ERROR(s, args...)		do { if (!unsquash_opts->be_quiet) { fprintf(stderr, s, ## args); } } while(0)
#define EXIT_UNSQUASH(s, args...)	do { fprintf(stderr, "FATAL ERROR aborting: "s, ## args); exit(1); } while(0)

#define FALSE 0

struct hash_table_entry {
  int	start;
  int	bytes;
  struct hash_table_entry *next;
};

typedef struct squashfs_operations {
  struct dir *(*squashfs_opendir)(char *pathname, unsigned int block_start, unsigned int offset);
  char *(*read_fragment)(unsigned int fragment);
  void (*read_fragment_table)();
  void (*read_block_list)(unsigned int *block_list, unsigned char *block_ptr, int blocks);
  struct inode *(*read_inode)(unsigned int start_block, unsigned int offset);
} squashfs_operations;

struct inode {
  int blocks;
  unsigned char *block_ptr;
  long long data;
  int fragment;
  int frag_bytes;
  gid_t gid;
  int inode_number;
  int mode;
  int offset;
  long long start;
  char symlink[65536];
  time_t time;
  int type;
  uid_t uid;
};

struct test {
  int mask;
  int value;
  int position;
  char mode;
};


struct path_entry {
	char *name;
	regex_t *preg;
	struct pathname *paths;
};

struct pathname {
	int names;
	struct path_entry *name;
};

struct pathnames {
	int count;
	struct pathname *path[0];
};
#define PATHS_ALLOC_SIZE 10


#define DIR_ENT_SIZE	16

struct dir_ent	{
	char		name[SQUASHFS_NAME_LEN + 1];
	unsigned int	start_block;
	unsigned int	offset;
	unsigned int	type;
};

struct dir {
	int		dir_count;
	int 		cur_entry;
	unsigned int	mode;
	uid_t		uid;
	gid_t		guid;
	unsigned int	mtime;
	struct dir_ent	*dirs;
};


/* Globally visible options */
int read_super (const char *, int);
void squashfs_stat (const char *);
void read_uids_guids (int);
void do_read_fragment_table (int);
void do_uncompress_inode_table (int);
void do_uncompress_directory_table (int);
int do_dir_scan (int, char *, struct pathnames *);

#ifdef USE_MMAP
/* In-mem ops */
int squashfs_parse_superblock (const unsigned char *);
#endif

#endif
