/*
 * Utility functions the image tools.
 *
 * Copyright (c) Simon Vallet 2008 <svallet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * utils.h
 */


#ifndef PATH_MAX
#define PATH_MAX 256
#endif

/**** Option parsing ****/
#define _KIMG_DO_NOTHING 0
#define _KIMG_DO_EXTRACT 1
#define _KIMG_DO_CREATE 2

/** bcm_kimg **/
struct kimg_opts {
  unsigned char todo;        /* One of the _KIMG_DO_* constants */
  unsigned char be_verbose; 
  char * dir;                /* Input or output directoryn depending on the action */
  char * raw_kernel;         /* raw (stripped) kernel file name */
  char * elf_kernel;         /* ELF kernel file name */
  char * lz_kernel;          /* LZMA-compressed kernel file name */
  char * kernel_img;         /* Broadcom kernel image, with header */
  off_t load_base;           /* Kernel load base */
};

int parse_kimg_opts (int, char **);

/** lzma_unsquash **/
struct unsquash_opts {
  unsigned char show_superblock_only;
  unsigned char do_list_files;
  unsigned char do_list_extended;
  unsigned char do_extract;
  unsigned char do_force_extract;
  unsigned char do_use_regex;
  unsigned char be_quiet;
  char *        output_dir;
};

int parse_unsquash_opts (int, char **);


/** bcm_fw_extract **/
struct extract_opts {
  char * output_dir;
  unsigned char dump_header;
  unsigned char dump_cfe;
  unsigned char dump_rootfs;
  unsigned char dump_kernel;
  unsigned char do_extract;
  unsigned char be_verbose;
};

int parse_fw_extract_opts (int, char **);

/** bcm_fw_create **/
struct create_opts {
  char * input_dir;
  char * board_id;
  char * chip_id;
  unsigned char include_bootldr;     /* Bootloader image, will not be included if NULL */
  unsigned char include_rootfs;      /* Root filesystem image, will not be included if NULL */
  unsigned char include_kernel;      /* Kernel image, will not be included if NULL */
  unsigned char include_bcm_hdr;     /* If 0, create an image which doesn't include a Broadcom header,
					for use with the 'flashimage' command */
};

int parse_fw_create_opts (int, char **);

/**** Misc utilities ****/
void hex_dump (const unsigned char *, size_t);
u_int32_t bcm_crc32 (const unsigned char *, size_t);
u_int32_t swap_bytes_32 (const u_int32_t);
int is_big_endian (void);


