/*
 * Unsquash a squashfs filesystem.  This is a highly compressed read only filesystem.
 *
 * Copyright (c) 2002, 2003, 2004, 2005, 2006, 2007
 *  Phillip Lougher <phillip@lougher.demon.co.uk>
 *
 * This file has been modified to support LZMA decompression as 
 * implemented in Broadcom chipsets -- most of the code is shamelessly 
 * borrowed from nb4-unsquash.c available at
 * svn://svn.gna.org/svn/openbox4/trunk/tools/nb4-unsquash
 *
 * Changes to this file are
 * Copyright (c) Simon Vallet <svallet@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * lzma_unsquash_main.c
 */
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <lzma_unsquash.h>
#include <utils.h>

extern struct unsquash_opts * unsquash_opts;

int main (int argc, char *argv[])
{
    
    int arg_i;
    const char * img_path; int img_fd;
    /* TODO: implement -d */
    char *dest = "squashfs-root";
    /* mmap -- TODO */
    /* void * img_ptr; struct stat img_st; */

    /* individual extraction -- TODO */
    struct pathnames *paths = NULL;
    /* struct pathname *path = NULL; */

    if ((arg_i = parse_unsquash_opts(argc, argv)) == -1) { return 1; }
    
    if ((geteuid() == SYSTEM_UID)) { umask(0); }

    img_path = argv[arg_i];

    /* Files to extract -- TODO */
    /*for (i = arg_i+1; i<argc; i++) { path = add_path(path, argv[i], argv[i]); }*/
    
    if ((img_fd = open(img_path, O_RDONLY)) != -1) {
      /* mmap() -- TODO */
      /*
      if (fstat(img_fd, &img_st) != -1) { 
	if ((img_ptr=mmap(NULL, img_st.st_size, PROT_READ, MAP_PRIVATE, img_fd, 0)) != MAP_FAILED) {
	  if ((err = squashfs_parse_superblock(img_ptr)) >= 0) {

	    if (unsquash_opts->show_superblock_only) { // We're done
	      squashfs_stat(img_path);
            } else { // Parse the rest 
	      img_ptr += err;
	      err = squashfs_parse_metatdata(img_ptr);


	    } 
	  } else {
	    fprintf(stderr, "Fatal error while parsing the superblock. Exiting.\n");
	  }
	  munmap(img_ptr, img_st.st_size);
	}
      }
      */
      if (read_super(img_path, img_fd) == FALSE) { return 1; }
      else {
	if (unsquash_opts->show_superblock_only) {
	  squashfs_stat(img_path);
	  return 0;
	}
      }
      read_uids_guids(img_fd);
      do_read_fragment_table(img_fd); /*s_ops.read_fragment_table(img_fd);*/ 
      /* uncompress_inode_table(img_fd, sBlk.inode_table_start, sBlk.directory_table_start); */
      do_uncompress_inode_table(img_fd);
      /* uncompress_directory_table(sBlk.directory_table_start, sBlk.fragment_table_start); */
      do_uncompress_directory_table(img_fd);
      
      /*
	if(path) {
		paths = init_subdir();
		paths = add_subdir(paths, path);
	}
      */


      do_dir_scan(img_fd, dest, paths);
      /* TODO
	 if (unsquash_opts->do_extract) {
	 printf("\n");
	 printf("created %d files\n", file_count);
	 printf("created %d directories\n", dir_count);
	 printf("created %d symlinks\n", sym_count);
	 printf("created %d devices\n", dev_count);
	 printf("created %d fifos\n", fifo_count);
	 }
      */


      close(img_fd);
    }

    return 0;
}
